package AboutWindow;
use Tk 8.0;

sub new {
	my $self = {};

	$self->{TEXT} = undef;
	$self->{TITLE} = undef;
	$self->{GEOMETRY} = undef;
	$self->{XCOORD} = undef;
	$self->{YCOORD} = undef;

	bless($self);
	return $self;
}

sub text {
	my $self = shift;
	$self->{TEXT} = shift if @_;
	return $self->{TEXT};
}

sub title {
	my $self = shift;
	$self->{TITLE} = shift if @_;
	return $self->{TITLE};
}

sub geometry {
	my $self = shift;
	$self->{GEOMETRY} = shift if @_;
	return $self->{GEOMETRY};
}

sub xcoord {
	my $self = shift;
	$self->{XCOORD} = shift if @_;
	return $self->{XCOORD};
}

sub ycoord {
	my $self = shift;
	$self->{YCOORD} = shift if @_;
	return $self->{YCOORD};
}

sub display {
	my $self = shift;
	my $mainWindow = shift;
	my $aboutMenuItem = shift if (@_);

	if ($aboutMenuItem) {
		$aboutMenuItem->configure(-state => 'disabled');
	}
	my $aboutWindow = $mainWindow->Toplevel();
	$aboutWindow->title($self->{TITLE});
	$aboutWindow->focus();
	$aboutWindow->resizable(0,0);

	my $x = $mainWindow->rootx();
	my $y = $mainWindow->rooty();
	my $xCoord = $x + $self->{XCOORD};
	my $yCoord = $y + $self->{YCOORD};
	$aboutWindow->geometry($self->{GEOMETRY}."+$xCoord+$yCoord");

	my $topFrame = $aboutWindow->Frame()
		->pack(-side => 'top');
	my $bottomFrame = $aboutWindow->Frame()
		->pack(-side => 'bottom');
	
	my $aboutFrame = $topFrame->Frame()
		->pack(-side => 'left', -padx => 15, -pady => 10);
	my $aboutLabel = $aboutFrame->Label(
		-text => $self->{TEXT}, 
		-justify => 'left')
		->pack(-side => 'top');
	
	my $closeButton = $bottomFrame->Button(
		-text => "Close", 
		-underline => 0, 
		-command => sub {
			if ($aboutMenuItem) {
				$aboutMenuItem->configure(-state => 'active');
			}
			$aboutWindow->destroy(); 
		}, 
		-width => 6)
		->pack(-side => 'top');
	$aboutWindow->bind("<Alt-Key-c>" => sub {
		if ($aboutMenuItem) {
			$aboutMenuItem->configure(-state => 'active');
		}
		$aboutWindow->destroy(); 
	});
}

1;

=head1 NAME

AboutWindow.pm

=head1 DESCRIPTION

AboutWindow.pm is a Perl module designed to allow easy creation of
an about window for Perl/Tk applications

=head1 PREREQUISITE

requires Tk 8.0

=head1 USAGE

B<$aboutwindow = new AboutWindow;>

=over 4

=item

creates a new AboutWindow object

=back

B<$aboutwindow>->B<text(I<string>);>

=over 4

=item

assigns a blurb of text to the AboutWindow object

=back

B<$aboutwindow>->B>title(I<string>);>

=over 4

=item

assigns a title to the AboutWindow object

=back

B<$aboutwindow>->B<geometry(I<string>);>

=over 4

=item

sets the size of the AboutWindow object (WIDTHxHEIGHT)

=back

B<$aboutwindow>->B<xcoord(I<integer>);>

=over 4

=item

sets the xcoordinate offset of the AboutWindow object from the root window

=back

B<$aboutwindow>->B<ycoord(I<integer>);>

=over 4

=item

sets the ycoordinate offset of the AboutWindow object from the root window

=back

B<$aboutwindow>->B<display;>

=over 4

=item

draws the AboutWindow object

=back

=head1 OSNAMES

Win32, Unix

=head1 SCRIPT CATEGORIES

GUI Programming

=cut
