package ConfigFile;

sub createConfig {
	my $configref = shift;
	my %config = %$configref;
	my $configdir = shift;
	my $statuscfg = shift;

        if ( ! -e $configdir ) {
                mkdir $configdir or die "Couldn't create config directory $configdir: $!\n";
        } elsif (! -d $configdir ) {
                die "$configdir isn't a directory, quitting.\n";
        }
  
	open (CONFIG, ">$statuscfg")
  		|| die "Cannot open file $statuscfg : $!\n";

	print CONFIG "<opt>\n";
	foreach my $key (keys(%config)) {
		print CONFIG "  <config ".$key."=\"".$config{$key}."\" />\n";
	}
	print CONFIG "</opt>\n";

	close (CONFIG);
}

sub readConfig {
	my $statuscfg = shift;

	my %config;
	open (CONFIG, "<$statuscfg")
		|| die "Cannot open file $statuscfg : $!\n";
	while (<CONFIG>) {
		if (/<config (.*?)="(.*?)"/) {
			$config{$1} = $2;
		}
	}
	close (CONFIG);
	return %config;
}

sub writeConfig {
	my $configref = shift;
	my %config = %$configref;
	my $statuscfg = shift;

	open (CONFIG, ">$statuscfg")
		|| die "Cannot open file $statuscfg $!\n";

	print CONFIG "<opt>\n";
	foreach my $key (keys(%config)) {
		$config{$key} = "" if !defined($config{$key});
		print CONFIG "  <config ".$key."=\"";
		print CONFIG $config{$key}."\" />\n";
	}
	print CONFIG "</opt>\n";

	close (CONFIG);
}

1;

=head1 NAME

ConfigFile.pm

=head1 DESCRIPTION

ConfigFile.pm is a perl module designed to allow easy manipulation of a configuration file for applications (xml format)

=head1 USAGE

B<ConfigFile::createConfig(hashref, string, string);>

=over 4

=item

Takes (in order) the configuration hash, directory that the config file will be saved in, and the config file name. Creates the config file.

=back

B<%hash = ConfigFile::readConfig(string);>

=over 4

=item

Takes the config file name. Reads the config file into a hash, which is then returned.

=back

B<ConfigFile::writeConfig(hashref, string);>

=over 4

=item

Takes the configuration hash, and the config file name. Writes the configuration hash to the config file.

=back

Format of the config file is:

	<opt>
	  <config KEY1="VALUE1" />
	  <config KEY2="VALUE2" />
	</opt>

=head1 OSNAMES

Win32, Unix

=head1 SCRIPT CATEGORIES

File IO

=cut
