#!/usr/bin/perl -w

##################################################
# Thanks to Rens Houben <shadur@systemec.nl> for changes
# to the way configuration files are handled, so they are
# stored in ~/.status, rather than the current directory
##################################################

use strict;
use English;
use IO::Socket;
use Net::FTP;
use Net::Ping;
use Net::SMTP;
use Tk 8.0;
use Tk::BrowseEntry;
use Tk::Checkbutton;
use Tk::Menubutton;
use Tk::Pane;
use Tk::Radiobutton;

use AboutWindow;
use ConfigFile;

#---------------------------------------------------
# Begin variable declaration
#---------------------------------------------------
my $version = "1.8.6";
my $homedir = $ENV{"HOME"};
my $configdir;
if (defined($homedir)) {
	$configdir = $homedir."/.status";
} else {
	$configdir = "/.status";
}
my $statuscfg = "$configdir/config";
my $servercfg = "$configdir/servers";
my $mainWindow;
my ($menuBar, $fileMenu, $updateMenuItem, $addMenuItem, $editMenuItem, $deleteMenuItem, $configMenuItem, $exitMenuItem, $helpMenu, $aboutMenuItem);
my ($labelFrame, $mainFrame, $mainPane, @listFrame);
my (@serverList, @portList, @statusList, @serverDelete, @emailSent);
my ($serverTitle, $portTitle, $statusTitle, $deleteTitle);
my ($buttonFrame, $updateButton, $addButton, $editButton, $deleteButton, $exitButton);
my (%config, %servers, %ports, %status, %type);
my @colors = qw( red blue green yellow black );

#---------------------------------------------------
# End variable declaration
#---------------------------------------------------

#---------------------------------------------------
# Begin main program
#---------------------------------------------------
# First off, see if there's a config file.  If not, create it.
#open (CONFIG, "<$statuscfg")
#	|| ConfigFile::createConfig($configdir, $statuscfg);
if (!(stat($statuscfg))[7]) {
	$config{color_up} = "green";
	$config{color_down} = "red";
	$config{refresh} = "60000";
        $config{server_timeout} = "2";
	$config{email_noti} = 0;
	$config{recip_address} = "";
	$config{sender_address} = "";
	$config{smtp_server} = "";
	ConfigFile::createConfig(\%config, $configdir, $statuscfg);
}

# Pull the servers and ports out of the XML file into a hash
my $fileError;
open (FH, "<$servercfg")
        or $fileError = 1;
if (!$fileError) {
        while (<FH>) {
		if (/<server address="(.*?)" name="(.*?)" port="(.*?)"/) {
                        $servers{$2} = $1;
                        $ports{$2} = $3;
			my $servername = $2;
			if (/type="(.*?)" \/>/) {
				$type{$servername} = $1;
			}
			if ((($ports{$servername} ne "") || (!$ports{$servername})) && ((!$type{$servername}) || ($type{$servername} eq ""))) {
				$type{$servername} = "port";
			}
                }
        }
}
close (FH);

# Just get the number of servers in the list so we can make the list
# boxes the right size
&resetEmailSent();

# Let's create the interface
#
# Create the main window where it will all be displayed
$mainWindow = MainWindow->new();
$mainWindow->title("Server Status");
# End main window creation

# Create a simple menubar with a File menu and a Help menu
$menuBar = $mainWindow->Frame(
	-relief => 'groove',
	-borderwidth => 2)
	->pack(-side => 'top', -fill => 'x');
# Create the File menu
$fileMenu = $menuBar->Menubutton(
	-text => 'File',
	-underline => 0,
	-tearoff => 0)
	->pack(-side => 'left');
$updateMenuItem = $fileMenu->command(
	-label => 'Update',
	-underline => 0,
	-command => sub { 
		\&updateServerList(); 
	});
$fileMenu->separator;
$addMenuItem = $fileMenu->command(
	-label => 'Add server', 
	-underline => 0, 
	-command => sub { 
		\&addServerWindow(); 
	});
$editMenuItem = $fileMenu->command(
	-label => 'Edit server',
	-underline => 0,
	-command => sub {
		\&editServer();
	});
$deleteMenuItem = $fileMenu->command(
	-label => 'Delete server', 
	-underline => 0, 
	-command => sub { 
		\&deleteServer(); 
	});
$fileMenu->separator;
$configMenuItem = $fileMenu->command(
	-label => 'Configure',
	-underline => 0,
	-command => sub {
		\&configWindow();
	});
$exitMenuItem = $fileMenu->command(
	-label => 'Exit', 
	-underline => 1, 
	-command => sub { 
		exit; 
	});
# Create the Help menu
$helpMenu = $menuBar->Menubutton(
	-text => 'Help', 
	-underline => 0, 
	-tearoff => 0)
	->pack(-side => 'right');
$aboutMenuItem = $helpMenu->command(
	-label => 'About', 
	-underline => 1, 
	-command => sub { 
		\&aboutWindow(); 
	});
# End menubar creation

# Create three frames... one for some labels, one for the list boxes, 
# and one for the buttons
$labelFrame = $mainWindow->Frame()
	->pack(-side => 'top', -anchor => 'w');
$mainFrame = $mainWindow->Frame()
	->pack(-side => 'top', -fill => 'y', -expand => 1);
$mainPane = $mainFrame->Scrolled(
	'Pane',
	-height => 220,
	-width => 325,
	-scrollbars => 'oe')
	->pack(-side => 'top', -fill => 'y', -expand => 1);
$buttonFrame = $mainWindow->Frame(
	-relief => 'groove', 
	-borderwidth => 2)
	->pack(-side => 'bottom', -pady => 10, -fill => 'x');
# End frame creation

# Create the list box labels
$serverTitle = $labelFrame->Label(
	-text => "Server", 
	-justify => 'left', 
	-width => 36)
	->pack(-side => 'left');
$portTitle = $labelFrame->Label(
	-text => "Port",
	-justify => 'right',
	-width => 7)
	->pack(-side => 'left');
$statusTitle = $labelFrame->Label(
	-text => "Status", 
	-justify => 'right', 
	-width => 6)
	->pack(-side => 'left');
# End list box label creation

# Create the Update, Add, and Exit buttons
$updateButton = $buttonFrame->Button(
	-text => "Update", 
	-underline => 0, 
	-command => sub { 
		\&updateServerList(); 
	}, 
	-width => 6)
	->pack(-side => 'left');
$addButton = $buttonFrame->Button(
	-text => "Add", 
	-underline => 0, 
	-command => sub { 
		\&addServerWindow(); 
	}, 
	-width => 6)
	->pack(-side => 'left');
$editButton = $buttonFrame->Button(
	-text => "Edit",
	-underline => 0,
	-command => sub {
		\&editServer();
	},
	-width => 6)
	->pack(-side => 'left');
$deleteButton = $buttonFrame->Button(
	-text => "Delete", 
	-underline => 0, 
	-command => sub { 
		\&deleteServer(); 
	}, 
	-width => 6)
	->pack(-side => 'left');

$exitButton = $buttonFrame->Button(
	-text => "Exit", 
	-underline => 1, 
	-command => sub { 
		exit; 
	}, 
	-width => 6)
	->pack(-side => 'right');
# End button creation

# Create some key bindings for the buttons and menus
$mainWindow->bind('<Alt-Key-u>' => sub { 
	\&updateServerList(); 
});
$mainWindow->bind('<Alt-Key-a>' => sub { 
	\&addServerWindow(); 
});
$mainWindow->bind('<Alt-Key-d>' => sub { 
	\&deleteServer(); 
});
$mainWindow->bind('<Alt-Key-c>' => sub {
	\&configWindow();
});
$mainWindow->bind('<Alt-Key-x>' => sub { 
	exit; 
});
$mainWindow->bind('<Alt-Key-b>' => sub { 
	\&aboutWindow(); 
});
$mainWindow->bind('<Alt-Key-f>' => sub { 
	$fileMenu->Post('','',0); 
	return; 
});
$mainWindow->bind('<Alt-Key-h>' => sub { 
	$helpMenu->Post('','',0); 
	return; 
});
# End key bindings
#
# End interface creation

# Read the config file
%config = ConfigFile::readConfig($statuscfg);
while (my($key,$value) = each %config) {
	if ($key =~ /color\d+/) {
		push(@colors, $value);
	}
}

# Fill in the list boxes with server info
drawServerList();

# Cause the server info list boxes to refresh
$mainWindow->repeat($config{refresh}, \&updateServerList);

MainLoop;
#---------------------------------------------------
# End main program
#---------------------------------------------------

#---------------------------------------------------
# Begin subroutines
#---------------------------------------------------
#---------------------------------------------------
# Simply displays an About window
# usage: &aboutWindow();
sub aboutWindow {
	my $text = " -= Server Status v".$version." =-\n\n";
	$text   .= "Written by     :\tMike Roessing\n";
	$text   .= "Email\t      :\txjar\@the-den.org\n";
	$text   .= "Address\t      :\thttp:\/\/www.the-den.org\n\n";
	$text   .= "Changes in v1.6.1 courtesy of\nRens Houben <shadur\@systemec.nl>";

	my $aboutWindow = new AboutWindow;
	$aboutWindow->text($text);
	$aboutWindow->title("About Server Status");
	$aboutWindow->xcoord("38");
	$aboutWindow->ycoord("10");
	$aboutWindow->geometry("300x165");
	$aboutWindow->display($mainWindow, $aboutMenuItem);
}

#---------------------------------------------------
# Add colors to the color list
# usage: &addColor($colorEntry, $browseEntry);
sub addColor {
	my($colorEntry, $browseEntry) = @_;

	my $color = $colorEntry->get;
	# If the entry box is empty, display an error message
	if (!$color) {
		&errorWindow("color");
	} else {
		my $numColors = $#colors-3;
	# Push the color into the @colors array
		push(@colors, $color);
	# Add the color to the config hash
		$config{"color".$numColors} = $color;
	# Ugh, I hate doing this to write just one line...
		ConfigFile::writeConfig(\%config, $statuscfg);
	# Add the color to the BrowseEntry widget
		$browseEntry->insert($numColors+4, $color);
	}
}

#---------------------------------------------------
# Allow the user to add their own colors without having to
# edit the config file by hand
# usage: &addColorWindow($configWindow, $browseEntry);
sub addColorWindow {
	my($configWindow,$browseEntry) = @_;
	my $addColorWindow = $mainWindow->Toplevel();
	$addColorWindow->title("Add color");
	$addColorWindow->focus();
	$addColorWindow->resizable(0,0);

	&positionWindow($configWindow, $addColorWindow, 11, 10, 150, 110);

	my $topFrame = $addColorWindow->Frame()
		->pack(-side => 'top');
	my $bottomFrame = $addColorWindow->Frame(
		-relief => 'groove',
		-borderwidth => 2)
		->pack(-side => 'top', -pady => 10, -fill => 'x');

	my $addColorLabel = $topFrame->Label(
		-text => "Enter a color to add\n(eg. \"#ff0000\" or \"red\")")
		->pack(-side => 'top');
	my $addColorEntry = $topFrame->Entry(
		-width => 10)
		->pack(-side => 'top');
	$addColorEntry->focus();

	my $addButton = $bottomFrame->Button(
		-text => "Add",
		-command => sub {
			&addColor($addColorEntry, $browseEntry);
			$addColorWindow->destroy();
		},
		-width => 6)
		->pack(-side => 'left');
	my $cancelButton = $bottomFrame->Button(
		-text => "Cancel",
		-command => sub {
			$addColorWindow->destroy();
		},
		-width => 6)
		->pack(-side => 'right');
}

#---------------------------------------------------
# Add the server and update the list
# usage: &addServer($serverNameEntry, $serverIPEntry, $serverPortEntry, $serverCheckType);
sub addServer {
	my ($serverNameEntry, $serverIPEntry, $serverPortEntry, $serverCheckType) = @_;
	my $serverName = $serverNameEntry->get;
	my $serverIP = $serverIPEntry->get;
	my $serverPort = $serverPortEntry->get;

# Check if the ip address entered fits the
# x(x(x)).x(x(x)).x(x(x)).x(x(x)) format
	if ((!$serverName) || (!$serverIP) || !($serverIP =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)) {
		&serverInfoError($serverName, $serverIP, $serverPort);
	} else {
# Delete the server list
		clearServerList();
# Add the server and port to the hash and XML file
		$servers{$serverName} = $serverIP;
		$ports{$serverName} = $serverPort;
		$type{$serverName} = $serverCheckType;
		&resetEmailSent();
		&writeServerList();
# Increment the server count and redraw the server list
		drawServerList();
# Delete the server info we typed in, ready to add another
		$serverNameEntry->delete(0, 'end');
		$serverIPEntry->delete(0, 'end');
		$serverPortEntry->delete(0, 'end');
	}
}

#---------------------------------------------------
# Allow the user to add a server to the list
# usage: &addServerWindow();
sub addServerWindow {
# Disable the Add button and menu item while the window is open
	$addButton->configure(-state => 'disabled');
	$addMenuItem->configure(-state => 'disabled');
# Draw the Add window
	my $addServerWindow = $mainWindow->Toplevel();
	$addServerWindow->title("Add a server");
	$addServerWindow->resizable(0,0);

	&positionWindow($mainWindow, $addServerWindow, 11, 10, 375, 225);

# Create one frame for the text entry boxes, another frame for the
# buttons
	my $topFrame = $addServerWindow->Frame()
		->pack(-side => 'top');
	my $bottomFrame = $addServerWindow->Frame(
		-relief => 'groove', 
		-borderwidth => 2)
		->pack(-side => 'top', -pady => 10, -fill => 'x');
# Draw the text entry boxes, with labels (give the Server Name entry
# focus)
	my $serverNameLabel = $topFrame->Label(
		-text => "Server Name")
		->pack(-side => 'top');
	my $serverNameEntry = $topFrame->Entry(
		-width => 50)
		->pack(-side => 'top');
	$serverNameEntry->focus();
	my $serverIPLabel = $topFrame->Label(
		-text => "Server IP Address")
		->pack(-side => 'top');
	my $serverIPEntry = $topFrame->Entry(
		-width => 50)
		->pack(-side => 'top');
	my $pingOrPortFrame = $topFrame->Frame()
		->pack(-side => 'top');
	my $pingOrPortLabel = $pingOrPortFrame->Label(
		-text => "Check status with ping or port connection?")
		->pack(-side => 'top');
	my $serverCheckType;
	my $pingFrame = $pingOrPortFrame->Frame()
		->pack(-side => 'left');
	my $pingRadiobutton = $pingFrame->Radiobutton(
		-variable => \$serverCheckType,
		-value => "ping")
		->pack(-side => 'left');
	my $pingLabel = $pingFrame->Label(
		-text => "Ping")
		->pack(-side => 'right');
	my $portFrame = $pingOrPortFrame->Frame()
		->pack(-side => 'right');
	my $portRadiobutton = $portFrame->Radiobutton(
		-variable => \$serverCheckType,
		-value => "port")
		->pack(-side => 'left');
	my $portLabel = $portFrame->Label(
		-text => "Port")
		->pack(-side => 'right');
	$portRadiobutton->select;
	my $serverPortLabel = $topFrame->Label(
		-text => "Server Port")
		->pack(-side => 'top');
	my $serverPortEntry = $topFrame->Entry(
		-width => 10,
		-state => 'normal',
		-background => 'white')
		->pack(-side => 'top');
	$pingRadiobutton->configure(-command => sub {
			if ((defined($ENV{USER}) ne "root") && ($OSNAME ne "MSWin32")) {
				&errorWindow("root");
				$portRadiobutton->select;
				$serverPortEntry->configure(-state => 'normal', -background => 'white');
				return 0;
			}
			$serverPortEntry->delete(0, 'end');
			$serverPortEntry->configure(-state => 'disabled', -background => 'grey');
		});
	$portRadiobutton->configure(-command => sub {
			$serverPortEntry->configure(-state => 'normal', -background => 'white');
		});

# Draw the buttons to add the server, clear the entered info, and close
# the Add window

	my $addServerButton = $bottomFrame->Button(
		-text => "Add", 
		-underline => 0, 
		-command => sub { 
			&addServer($serverNameEntry, $serverIPEntry, $serverPortEntry, $serverCheckType); 
			$serverNameEntry->focus(); 
		}, 
		-width => 6)
		->pack(-side => 'left');
	my $resetButton = $bottomFrame->Button(
		-text => "Reset", 
		-underline => 0, 
		-command => sub {
			&resetButtonSub("add", $serverNameEntry, $serverIPEntry, $serverPortEntry, $portRadiobutton);
		}, 
		-width => 6)
		->pack(-side => 'left');
	my $closeButton = $bottomFrame->Button(
		-text => "Close", 
		-underline => 0, 
		-command => sub { 
			&closeButtonSub($addServerWindow, $addMenuItem, $addButton);
		}, 
		-width => 6)
		->pack(-side => 'right');

# Create the bindings for the buttons

	$addServerWindow->bind('<Alt-Key-a>' => sub { 
		&addServer($serverNameEntry, $serverIPEntry, $serverPortEntry, $serverCheckType); 
		$serverNameEntry->focus(); 
	});
	$addServerWindow->bind('<Alt-Key-r>' => sub {
		&resetButtonSub("add", $serverNameEntry, $serverIPEntry, $serverPortEntry, $portRadiobutton);
	});
	$addServerWindow->bind('<Alt-Key-c>' => sub {
		&closeButtonSub($addServerWindow, $addMenuItem, $addButton);
	});
}

#---------------------------------------------------
# Pretty self explanatory... clears the list boxes of info then redraws
# them after a short delay (for that "refresh" look)
# usage: &clearServerList($deleted);
sub clearServerList {
# Ya, this is stupid, but it works...
        my ($deleted) = @_;
        if ($deleted) {
                for (0..(scalar(keys %servers))) {
        		$listFrame[$_]->destroy();
                }
        } else {
                for (0..(scalar(keys %servers))-1) {
                        $listFrame[$_]->destroy();
                }
        }
}

#---------------------------------------------------
# Simple sub to cut down on lines of code...
# usage: &closeButtonSub($childWindow, $parentMenuItem [, $parentButton]);
sub closeButtonSub {
	my ($childWindow, $parentMenuItem, $parentButton) = @_;
	$childWindow->destroy();
	$parentButton->configure(-state => 'active') if defined($parentButton);
	$parentMenuItem->configure(-state => 'active');
}

#---------------------------------------------------
# Called when &configWindow() is called, and also when the
# configEmailCheckbutton and configWriteHTMLCheckbutton is clicked
# usage: God, I can't remember why I did this sub the way I did... I won't even bother trying
sub configBoxes {
	my @vars = @_;
        my $state = shift(@vars);

	my $doColor;
	if ($vars[$#vars] eq "check") {
		$doColor = 0;
		pop(@vars);
	} else {
		$doColor = 1;
	}
# If the checkbox is not checked, disable and grey out the entry boxes
	if (!$state) {
                foreach my $entryBox (@vars) {
        		$entryBox->configure(
        			-state => 'disabled',
        			-background => 'grey') if $doColor;
			$entryBox->configure(
				-state => 'disabled') if !$doColor;
                }
# If the checkbox is checked, enable the entry boxes
	} else {
                foreach my $entryBox (@vars) {
        		$entryBox->configure(
        			-state => 'normal',
        			-background => 'white') if $doColor;
			$entryBox->configure(
				-state => 'normal') if !$doColor;
                }
	}
}

#---------------------------------------------------
# Simply displays the Configuration window
# usage: &configWindow();
sub configWindow {
	$configMenuItem->configure(-state => 'disabled');

	my $configWindow = $mainWindow->Toplevel();
	$configWindow->title("Server Status Configuration");
	$configWindow->focus();
	$configWindow->resizable(0,0);

	&positionWindow($mainWindow, $configWindow, 20, 10, 325, 400);

# Temp variables for the colors, if the user hits cancel,
# these will be used to revert back to original colors
	my $tempConfigUpColor = $config{color_up};
	my $tempConfigDownColor = $config{color_down};

# Make a top and bottom frame, top for the config info,
# bottom for the Close button
	my $topFrame = $configWindow->Frame()
		->pack(-side => 'top');
	my $bottomFrame = $configWindow->Frame(
		-relief => 'groove',
		-borderwidth => 2)
		->pack(-side => 'bottom', -pady => 10, -fill => 'x');

# Make the frames for each option setting
	my $displayFrame = $topFrame->Frame(
		-relief => 'groove',
		-borderwidth => 2)
		->pack(-side => 'top', -fill => 'x');
	my $emailFrame = $topFrame->Frame(
		-relief => 'groove',
		-borderwidth => 2)
		->pack(-side => 'top', -fill => 'x');

        my $HTMLFrame = $topFrame->Frame(
                -relief => 'groove',
                -borderwidth => 2)
                ->pack(-side => 'top', -fill => 'x');

	my $configUpFrame = $displayFrame->Frame()
		->pack(-side => 'top', -fill => 'x');
	my $configDownFrame = $displayFrame->Frame()
		->pack(-side => 'top', -fill => 'x');
	my $configRefreshFrame = $displayFrame->Frame()
		->pack(-side => 'top', -fill => 'x');
	my $configServerTimeoutFrame = $displayFrame->Frame()
                ->pack(-side => 'top', -fill => 'x');

	my $configEmailCheckFrame = $emailFrame->Frame(-relief => 'raised', -borderwidth => 1)
		->pack(-side => 'top');
	my $configRecipAddressFrame = $emailFrame->Frame()
		->pack(-side => 'top', -fill => 'x');
	my $configSenderAddressFrame = $emailFrame->Frame()
		->pack(-side => 'top', -fill => 'x');
	my $configSMTPServerFrame = $emailFrame->Frame()
		->pack(-side => 'top', -fill => 'x');

        my $configWriteHTMLCheckFrame = $HTMLFrame->Frame(-relief => 'raised', -borderwidth => 1)
                ->pack(-side => 'top');
	my $configUseFTPCheckFrame = $HTMLFrame->Frame(-relief => 'raised', -borderwidth => 1)
		->pack(-side => 'top');
        my $configFTPHostFrame = $HTMLFrame->Frame()
                ->pack(-side => 'top', -fill => 'x');
        my $configFTPDirFrame = $HTMLFrame->Frame()
                ->pack(-side => 'top', -fill => 'x');
        my $configFTPUserFrame = $HTMLFrame->Frame()
                ->pack(-side => 'top', -fill => 'x');
        my $configFTPPassFrame = $HTMLFrame->Frame()
                ->pack(-side => 'top', -fill => 'x');

# Add the label and optionmenu for the Up color
	my $configUpLabel = $configUpFrame->Label(
		-text => "Up status color:\t",
		-justify => 'left',
		-width => 25)
		->pack(-side => 'left');
	my $configUpBrowseEntry;
	my $configUpButton = $configUpFrame->Button(
		-text => "Add",
		-width => 3,
		-command => sub {
			&addColorWindow($configWindow, $configUpBrowseEntry);
		})
		->pack(-side => 'right', -padx => 5);
	$configUpBrowseEntry = $configUpFrame->BrowseEntry(
		-variable => \$config{color_up},
		-choices => \@colors,
		-listwidth => 20,
		-width => 8)
		->pack(-side => 'right');


# Add the label and optionmenu for the Down color
	my $configDownLabel = $configDownFrame->Label(
		-text => "Down status color:\t",
		-justify => 'left',
		-width => 25)
		->pack(-side => 'left');
	my $configDownBrowseEntry;
	my $configDownButton = $configDownFrame->Button(
		-text => "Add",
		-width => 3,
		-command => sub {
			&addColorWindow($configWindow, $configDownBrowseEntry);
		})
		->pack(-side => 'right', -padx => 5);
	$configDownBrowseEntry = $configDownFrame->BrowseEntry(
		-variable => \$config{color_down},
		-choices => \@colors,
		-listwidth => 20,
		-width => 8)
		->pack(-side => 'right');

# Add the label and entry box for the refresh rate
	my $refreshSeconds = $config{refresh} / 1000;
	my $configRefreshLabel = $configRefreshFrame->Label(
		-text => "Refresh (seconds):\t",
		-justify => 'left',
		-width => 25)
		->pack(-side => 'left');
	my $configRefreshEntry = $configRefreshFrame->Entry(
		-width => 5,
                -justify => 'right',
		-textvariable => \$refreshSeconds)
		->pack(-side => 'right');

# Add the label and entrybox for the server timeout
        my $configServerTimeoutLabel = $configServerTimeoutFrame->Label(
                -text => "Server Timeout (seconds):\t",
                -justify => 'left',
                -width => 25)
                ->pack(-side => 'left');
        my $configServerTimeoutEntry = $configServerTimeoutFrame->Entry(
                -width => 5,
                -justify => 'right',
                -textvariable => \$config{server_timeout})
                ->pack(-side => 'right');

# Add the label and checkbox for email notification
	my $tempEmailNoti = $config{email_noti};
	my $configEmailCheckLabel = $configEmailCheckFrame->Label(
		-text => "Email Notification?\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configEmailCheckbutton = $configEmailCheckFrame->Checkbutton(
		-variable => \$config{email_noti})
		->pack(-side => 'right');

# Add the label and textbox for the recipient email address
	my $tempRecipAddress = $config{recip_address};
	my $configRecipAddressLabel = $configRecipAddressFrame->Label(
		-text => "Recipient Address:\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configRecipAddressEntry = $configRecipAddressFrame->Entry(
		-width => 20,
                -justify => 'right',
		-textvariable => \$config{recip_address})
		->pack(-side => 'right');

# Add the label and textbox for the sender email address
	my $tempSenderAddress = $config{sender_address};
	my $configSenderAddressLabel = $configSenderAddressFrame->Label(
		-text => "Sender Address:\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configSenderAddressEntry = $configSenderAddressFrame->Entry(
		-width => 20,
                -justify => 'right',
		-textvariable => \$config{sender_address})
		->pack(-side => 'right');

# Add the label and textbox for the smtp server
	my $tempSMTPServer = $config{smtp_server};
	my $configSMTPServerLabel = $configSMTPServerFrame->Label(
		-text => "SMTP Server:\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configSMTPServerEntry = $configSMTPServerFrame->Entry(
		-width => 20,
                -justify => 'right',
		-textvariable => \$config{smtp_server})
		->pack(-side => 'right');

	$configEmailCheckbutton->configure(
		-command => sub {
			&configBoxes($config{email_noti}, $configRecipAddressEntry, $configSenderAddressEntry, $configSMTPServerEntry);
		});

# Enable or disable the email configuration boxes based
# on whether email notification is set or not
	&configBoxes($config{email_noti}, $configRecipAddressEntry, $configSenderAddressEntry, $configSMTPServerEntry);

# Add the label and checkbox for HTML usage
	my $tempWriteHTML = $config{writeHTML};
	my $configWriteHTMLLabel = $configWriteHTMLCheckFrame->Label(
		-text => "Write HTML File?\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configWriteHTMLCheckbutton = $configWriteHTMLCheckFrame->Checkbutton(
		-variable => \$config{writeHTML})
		->pack(-side => 'right');

	my $tempUseFTP = $config{useFTP};
	my $configUseFTPLabel = $configUseFTPCheckFrame->Label(
		-text => "Upload HTML via FTP?\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configUseFTPCheckbutton = $configUseFTPCheckFrame->Checkbutton(
		-variable => \$config{useFTP})
		->pack(-side => 'right');

# Add the label and entry for the FTP server
        my $tempFTPHost = $config{ftp_host};
        my $configFTPHostLabel = $configFTPHostFrame->Label(
		-text => "FTP Server:\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configFTPHostEntry = $configFTPHostFrame->Entry(
		-width => 20,
                -justify => 'right',
		-textvariable => \$config{ftp_host})
		->pack(-side => 'right');

# Add the label and entry for the FTP directory
        my $tempFTPDir = $config{ftp_dir};
        my $configFTPDirLabel = $configFTPDirFrame->Label(
                -text => "FTP Directory:\t",
                -justify => 'left',
                -width => 20)
                ->pack(-side => 'left');
        my $configFTPDirEntry = $configFTPDirFrame->Entry(
                -width => 20,
                -justify => 'right',
                -textvariable => \$config{ftp_dir})
                ->pack(-side => 'right');

# Add the label and entry for the FTP username
        my $tempFTPUser = $config{ftp_user};
        my $configFTPUserLabel = $configFTPUserFrame->Label(
		-text => "FTP Username:\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configFTPUserEntry = $configFTPUserFrame->Entry(
		-width => 20,
                -justify => 'right',
		-textvariable => \$config{ftp_user})
		->pack(-side => 'right');
  
# Add the label and entry for the FTP Password
        my $tempFTPPass = $config{ftp_pass};
        my $configFTPPassLabel = $configFTPPassFrame->Label(
		-text => "FTP Password:\t",
		-justify => 'left',
		-width => 20)
		->pack(-side => 'left');
	my $configFTPPassEntry = $configFTPPassFrame->Entry(
		-width => 20,
                -justify => 'right',
                -show => '*',
		-textvariable => \$config{ftp_pass})
		->pack(-side => 'right');

        $configWriteHTMLCheckbutton->configure(
                -command => sub {
                        &configBoxes($config{writeHTML}, $configUseFTPCheckbutton, "check");
                });
	$configUseFTPCheckbutton->configure(
		-command => sub {
			&configBoxes($config{useFTP}, $configFTPDirEntry, $configFTPHostEntry, $configFTPUserEntry, $configFTPPassEntry);
		});

# Enable or disable the HTML configuration boxes based
# on whether HTML output is set or not
        &configBoxes($config{writeHTML}, $configUseFTPCheckbutton, "check");
	&configBoxes($config{useFTP}, $configFTPDirEntry, $configFTPHostEntry, $configFTPUserEntry, $configFTPPassEntry);
        
# Put the saveButton in the bottomFrame
# Will write the new settings to the config file, close
# the options window, then refresh the server list
	my $saveButton = $bottomFrame->Button(
		-text => "Save", 
		-underline => 0, 
		-command => sub {
			$config{refresh} = $configRefreshEntry->get * 1000;
			ConfigFile::writeConfig(\%config, $statuscfg);
			$configMenuItem->configure(-state => 'active');
			$configWindow->destroy();
			&updateServerList();
		}, 
		-width => 6)
		->pack(-side => 'left');

# Put the cancelButton in the bottomFrame
# Will revert the colors back to original, revert
# the email settings back to original, close the
# options window, then refresh the server list
	my $cancelButton = $bottomFrame->Button(
		-text => "Cancel",
		-underline => 0,
		-command => sub {
			$config{color_up} = $tempConfigUpColor || "";
			$config{color_down} = $tempConfigDownColor || "";
			$config{email_noti} = $tempEmailNoti || "";
			$config{recip_address} = $tempRecipAddress || "";
			$config{sender_address} = $tempSenderAddress || "";
			$config{smtp_server} = $tempSMTPServer || "";
                        $config{writeHTML} = $tempWriteHTML || "";
			$config{useFTP} = $tempUseFTP || "";
                        $config{ftp_host} = $tempFTPHost || "";
                        $config{ftp_user} = $tempFTPUser || "";
                        $config{ftp_pass} = $tempFTPPass || "";
			&closeButtonSub($configWindow, $configMenuItem);
			#$configMenuItem->configure(-state => 'active');
			#$configWindow->destroy();
			&updateServerList();
		},
		-width => 6)
		->pack(-side => 'right');

# Same as hitting the saveButton
	$configWindow->bind("<Alt-Key-s>" => sub {
		$config{refresh} = $configRefreshEntry->get * 1000;
		ConfigFile::writeConfig(\%config, $statuscfg);
		$configMenuItem->configure(-state => 'active');
		$configWindow->destroy();
		&updateServerList();
	});
# Same as hitting the cancelButton
	$configWindow->bind("<Alt-Key-c>" => sub {
		$config{color_up} = $tempConfigUpColor;
		$config{color_down} = $tempConfigDownColor;
		$configMenuItem->configure(-state => 'active');
		$configWindow->destroy();
		&updateServerList();
	});
}

#---------------------------------------------------
# Delete the server(s) selected and update the list
# usage: &deleteServer();
sub deleteServer {
	my $deleted = 0;
	my $i = 0;
# Ugh, I hate having to do it this way, but whatever
	foreach my $serverName (sort(keys %servers)) {
# If the serverListbox is selected...
		my @selected = $serverList[$i]->curselection;
		if (defined($selected[0]) && $selected[0] == 0) {
# grab the server name...
			my $serverToDelete = $serverList[$i]->get(0);
# delete the server and from the hash...
			delete $servers{$serverToDelete};
			delete $ports{$serverToDelete};
			delete $type{$serverToDelete};
			$deleted++;
			&resetEmailSent();
		}
		$i++;
	}
	if ($deleted == 0) {
		&errorWindow("delete");
	} else {
		&writeServerList();
# and update the server list and number of servers we have
		&updateServerList($deleted);
	}
}

#---------------------------------------------------
# Fills the list boxes with the server name and status (Up or Down)
# usage: &drawServerList();
sub drawServerList {
# Draw the list boxes for the server and status info, each set in 
# their own frame
	my $i = 0;
	foreach my $server (keys %servers) {
		$listFrame[$i] = $mainPane->Frame()
			->pack(-side => 'top');
		$serverList[$i] = $listFrame[$i]->Listbox(
			-background => 'white', 
			-height => 1, 
			-width => 34)
			->pack(-side => 'left');
		$portList[$i] = $listFrame[$i]->Listbox(
			-background => 'white',
			-height => 1,
			-width => 6)
			->pack(-side => 'left');
		$statusList[$i] = $listFrame[$i]->Listbox(
			-background => 'white', 
			-height => 1, 
			-width => '6')
			->pack(-side => 'left');
		$i++;
	}

	$i = 0;
	foreach my $server (sort(keys %servers)) {
# Put the server name in its list box
		$serverList[$i]->insert('end', $server);
		$portList[$i]->insert('end', $ports{$server});
		if ($type{$server} eq "port") {
# Open a socket connection to the specified port on each ip address
	                my $s = IO::Socket::INET->new(
        	                PeerHost => $servers{$server},
                	        PeerPort => $ports{$server},
                        	Timeout => $config{server_timeout},
	                        Proto => 'tcp');

        	        if ($s) {
# If the socket is created, color the text the up color and the
# status is "Up"
                	        close ($s);
				my $laststatus = 1;
				$laststatus = $status{$server} if defined($status{$server});
                        	$status{$server} = 1;
	                        $statusList[$i]->configure(-background => $config{color_up});
				if (($laststatus != $status{$server}) && ($config{email_noti})) {
					&sendEmailNoti($server, "back up");
				}
        	        } else {
# If the socket is not created, color the text the down color and
# the status is "Down"
				my $laststatus = 0;
				$laststatus = $status{$server} if defined($status{$server});
                	        $status{$server} = 0;
                        	$statusList[$i]->configure(-background => $config{color_down});
	                        if ((($emailSent[$i] == 0) || ($laststatus != $status{$server})) && ($config{email_noti})) {
					$emailSent[$i] = &sendEmailNoti($server, "DOWN");
                	        }
	                }
		} elsif ($type{$server} eq "ping") {
			my $p = Net::Ping->new("icmp");
			if ($p->ping($servers{$server}, 2)) {
				$p->close();
				my $laststatus = 1;
				$laststatus = $status{$server} if defined($status{$server});
				$status{$server} = 1;
				$statusList[$i]->configure(-background => $config{color_up});
				if (($laststatus != $status{$server}) && ($config{email_noti})) {
					&sendEmailNoti($server, "back up");
				}
			} else {
				my $laststatus = 0;
				$laststatus = $status{$server} if defined($status{$server});
				$status{$server} = 0;
				$statusList[$i]->configure(-background => $config{color_down});
				if ((($emailSent[$i] == 0) || ($laststatus != $status{$server})) && ($config{email_noti})) {
					$emailSent[$i] = &sendEmailNoti($server, "DOWN");
				}
			}
		}
# Update the main window after each server so the program doesn't 
# look like it has stalled
                $mainWindow->update;
                $i++;
	}
        &writeHTML() if $config{writeHTML};
}

#---------------------------------------------------
# An error window in case you hit Delete/Edit without selecting a server
# usage: &errorWindow($errorType);
sub errorWindow {
	my ($errorType) = @_;
	my $errorText = "";
	if (($errorType eq "edit") || ($errorType eq "delete")) {
		$errorText = "\nYou must select a server to ".$errorType."!\n";
	} elsif ($errorType eq "smtp") {
		$errorText = "\nCould not connect to SMTP server\n\t".$config{smtp_server}."\n";
	} elsif ($errorType eq "color") {
		$errorText = "\nYou must enter a color to add!\n";
	} elsif ($errorType eq "root") {
		$errorText = "\nYou must be root to select this option!\n";
	}

	my $errorWindow = $mainWindow->Toplevel();
	$errorWindow->title("Error");
	$errorWindow->focus();
	$errorWindow->resizable(0,0);

	&positionWindow($mainWindow, $errorWindow, 38, 10, 230, 90);

# Make a top and bottom frame, top for the picture and About info,
# bottom for the Done button
	my $topFrame = $errorWindow->Frame()
		->pack(-side => 'top');
	my $bottomFrame = $errorWindow->Frame()
		->pack(-side => 'bottom');
	
# And now make a frame for the About info inside the topFrame
	my $messageLabel = $topFrame->Label(
		-text => $errorText, 
		-justify => 'left')
		->pack(-side => 'top');
	
# Put the doneButton in the bottomFrame
	my $closeButton = $bottomFrame->Button(
		-text => "Close", 
		-underline => 0, 
		-command => sub { 
			$errorWindow->destroy(); 
		}, 
		-width => 6)
		->pack(-side => 'top');
	$errorWindow->bind("<Alt-Key-c>" => sub { 
		$errorWindow->destroy(); 
	});
}

#---------------------------------------------------
# The logic for determining which server is to be edited
# usage: &editServer();
sub editServer {
	my $i = 0;
	my $edited = 0;
	foreach my $serverName (sort(keys %servers)) {
# If the serverListbox is selected...
		my @selected = $serverList[$i]->curselection;
		if (defined($selected[0]) && $selected[0] == 0) {
# grab the server name...
			my $serverToEdit = $serverList[$i]->get(0);
# and open the edit server window with the server information
			&editServerWindow($serverToEdit, $i);
			$edited = 1;
			last;
		}
		$i++;
	}
	&errorWindow("edit") if !$edited;
}

#---------------------------------------------------
# Simply allows the user to edit an existing server listing
# usage: &editServerWindow($serverToEdit, $index);
sub editServerWindow {
	my ($serverToEdit, $index) = @_;
# Disable the Add button and menu item while the window is open
	$editButton->configure(-state => 'disabled');
	$editMenuItem->configure(-state => 'disabled');
# Draw the Add window
	my $editServerWindow = $mainWindow->Toplevel();
	$editServerWindow->title("Edit a server");
	$editServerWindow->focus();
	$editServerWindow->resizable(0,0);

	&positionWindow($mainWindow, $editServerWindow, 11, 10, 375, 225);

# Create one frame for the text entry boxes, another frame for the
# buttons
	my $topFrame = $editServerWindow->Frame()
		->pack(-side => 'top');
	my $bottomFrame = $editServerWindow->Frame(
		-relief => 'groove', 
		-borderwidth => 2)
		->pack(-side => 'top', -pady => 10, -fill => 'x');
# Draw the text entry boxes, with labels (give the Server Name entry
# focus)
	my $serverNameLabel = $topFrame->Label(
		-text => "Server Name")
		->pack(-side => 'top');
	my $serverNameEntry = $topFrame->Entry(
		-width => 50)
		->pack(-side => 'top');
	$serverNameEntry->insert('end', $serverToEdit);
	my $serverIPLabel = $topFrame->Label(
		-text => "Server IP Address")
		->pack(-side => 'top');
	my $serverIPEntry = $topFrame->Entry(
		-width => 50)
		->pack(-side => 'top');
	$serverIPEntry->insert('end', $servers{$serverToEdit});
	my $serverCheckType = $type{$serverToEdit};
	my $pingOrPortFrame = $topFrame->Frame()
		->pack(-side => 'top');
	my $pingOrPortLabel = $pingOrPortFrame->Label(
		-text => "Check status with ping or port connection?")
		->pack(-side => 'top');
	my $pingFrame = $pingOrPortFrame->Frame()
		->pack(-side => 'left');
	my $pingRadiobutton = $pingFrame->Radiobutton(
		-variable => \$serverCheckType,
		-value => "ping")
		->pack(-side => 'left');
	my $pingLabel = $pingFrame->Label(
		-text => "Ping")
		->pack(-side => 'right');
	my $portFrame = $pingOrPortFrame->Frame()
		->pack(-side => 'right');
	my $portRadiobutton = $portFrame->Radiobutton(
		-variable => \$serverCheckType,
		-value => "port")
		->pack(-side => 'left');
	my $portLabel = $portFrame->Label(
		-text => "Port")
		->pack(-side => 'right');
	my $serverPortLabel = $topFrame->Label(
		-text => "Server Port")
		->pack(-side => 'top');
	my $serverPortEntry = $topFrame->Entry(
		-width => 10)
		->pack(-side => 'top');
	if ($serverCheckType eq "ping") {
		$pingRadiobutton->select;
		$serverPortEntry->configure(-background => 'grey', -state => 'disabled');
	} else {
		$portRadiobutton->select;
		$serverPortEntry->configure(-background => 'white', -state => 'normal');
		$serverPortEntry->insert('end', $ports{$serverToEdit});
	}
	$pingRadiobutton->configure(-command => sub {
			if ((defined($ENV{USER}) ne "root") && ($OSNAME ne "MSWin32")) {
				&errorWindow("root");
				$portRadiobutton->select;
				$serverPortEntry->configure(-state => 'normal', -background => 'white');
				return 0;
			}
			$serverPortEntry->delete(0, 'end');
			$serverPortEntry->configure(-state => 'disabled', -background => 'grey');
		});
	$portRadiobutton->configure(-command => sub {
			$serverPortEntry->configure(-state => 'normal', -background => 'white');
		});
# Draw the buttons to add the server, clear the entered info, and close
# the Add window
	my $editServerButton = $bottomFrame->Button(
		-text => "Save", 
		-underline => 0, 
		-command => sub {
			my $serverName = $serverNameEntry->get;
			my $serverIP = $serverIPEntry->get;
			my $serverPort = $serverPortEntry->get;
# If either the Server Name is blank, Server IP is blank or invalid,
# or Server Port is blank or invalid (nonnumeric)...
			if ((!$serverName) || (!$serverIP) || !($serverIP =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)) {
				&serverInfoError($serverName, $serverIP);
# otherwise...
			} else {
				&saveServerInfo($serverToEdit, $serverName, $serverIP, $serverPort, $serverCheckType);
				$editButton->configure(-state => 'active');
				$editServerWindow->destroy;
			}
		}, 
		-width => 6)
		->pack(-side => 'left');
# This button should revert settings back to original!
	my $resetButton = $bottomFrame->Button(
		-text => "Reset", 
		-underline => 0, 
		-command => sub {
			&resetButtonSub("edit", $serverNameEntry, $serverIPEntry, $serverPortEntry, $portRadiobutton, $serverToEdit, $pingRadiobutton, $type{$serverToEdit});
		}, 
		-width => 6)
		->pack(-side => 'left');
	my $closeButton = $bottomFrame->Button(
		-text => "Close", 
		-underline => 0, 
		-command => sub {
			&closeButtonSub($editServerWindow, $editMenuItem, $editButton);
		}, 
		-width => 6)
		->pack(-side => 'right');
# Create the bindings for the buttons
	$editServerWindow->bind('<Alt-Key-s>' => sub {
		my $serverName = $serverNameEntry->get;
		my $serverIP = $serverIPEntry->get;
		my $serverPort = $serverPortEntry->get;
		if ((!$serverName) || (!$serverIP) || !($serverIP =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)) {
			&serverInfoError($serverName, $serverIP, $serverPort);
		} else {
			&saveServerInfo($serverToEdit, $serverName, $serverIP, $serverPort, $serverCheckType);
			$editButton->configure(-state => 'active');
			$editServerWindow->destroy;
		} 
	});
	$editServerWindow->bind('<Alt-Key-r>' => sub {
		&resetButtonSub("edit", $serverNameEntry, $serverIPEntry, $serverPortEntry, $portRadiobutton, $serverToEdit, $pingRadiobutton, $type{$serverToEdit});
	});
	$editServerWindow->bind('<Alt-Key-c>' => sub {
		&closeButtonSub($editServerWindow, $editMenuItem, $editButton);
	});
}

#---------------------------------------------------
# Uploads the HTML file from &writeHTML to the FTP server specified in
# the configuration settings
# usage: &ftpHTML();
sub ftpHTML {
        my $ftp = Net::FTP->new($config{ftp_host}, Debug => 0)
		|| warn "Connection failed: $!\n";
        $ftp->login($config{ftp_user}, $config{ftp_pass})
		|| warn "Login failed: ".$ftp->message;
        $ftp->cwd($config{ftp_dir})
		|| warn "CWD failed: ".$ftp->message;
        $ftp->put("$configdir/status.html")
		|| warn "Could not put status.html: ".$ftp->message;
        $ftp->quit;
}

#---------------------------------------------------
# Simple sub to cut down on lines of code...
# Just positions the window nicely
# usage: &positionWindow($parentWindow, $childWindow, $xOffset, $yOffset, $width, $height);
sub positionWindow {
	my($parentWindow, $childWindow, $xOffset, $yOffset, $width, $height) = @_;
	my $x = $parentWindow->rootx();
	my $y = $parentWindow->rooty();
	my $xCoord = $x + $xOffset;
	my $yCoord = $y + $yOffset;
	$childWindow->geometry($width."x".$height."+".$xCoord."+".$yCoord);
}

#---------------------------------------------------
# Simple sub to cut down on lines of code...
# Revert back to original settings in the Add and Edit Server windows
# usage: &resetButtonSub($parent, $serverNameEntry, $serverIPEntry, $serverPortEntry, $portRadiobutton [, $serverToEdit, $pingRadiobutton, $serverCheckType]);
sub resetButtonSub {
	my ($parent, $serverNameEntry, $serverIPEntry, $serverPortEntry, $portRadiobutton, $serverToEdit, $pingRadiobutton, $serverCheckType) = @_;
	if ($parent eq "add") {
		$serverNameEntry->delete(0, 'end');
		$serverIPEntry->delete(0, 'end');
		$serverPortEntry->delete(0, 'end');
		$serverPortEntry->configure(-state => 'normal', -background => 'white');
		$portRadiobutton->select;
		$serverNameEntry->focus();
	} elsif ($parent eq "edit") {
		$serverNameEntry->delete(0, 'end');
		$serverNameEntry->insert('end', $serverToEdit);
		$serverIPEntry->delete(0, 'end');
		$serverIPEntry->insert('end', $servers{$serverToEdit});
		if ($serverCheckType eq "ping") {
			$pingRadiobutton->select;
			$serverPortEntry->delete(0, 'end');
			$serverPortEntry->configure(-background => 'grey', -state => 'disabled');
		} else {
			$portRadiobutton->select;
			$serverPortEntry->configure(-background => 'white', -state => 'normal');
			$serverPortEntry->insert('end', $ports{$serverToEdit});
		}
		$serverPortEntry->delete(0, 'end');
		$serverPortEntry->insert('end', $ports{$serverToEdit});
	}
}

#---------------------------------------------------
# Just reset the @emailSent list so there are no undefined list elements
# (required after a server is added/deleted)
# usage: &resetEmailSent();
sub resetEmailSent {
	my $i = 0;
	foreach (keys %servers) {
		$emailSent[$i] = 0;
        	$i++;
	}
	undef $i;
}

#---------------------------------------------------
# Display a helpful error window if invalid information is entered in the
# Add Server window
# usage: &serverInfoError($serverName, $serverIP, $serverPort);
sub serverInfoError {
	my ($serverName, $serverIP, $serverPort) = @_;
# Just draw a simple error window explaining the problem
	my $errorWindow = $mainWindow->Toplevel();
	$errorWindow->title("Error");
	$errorWindow->focus();
	$errorWindow->Label(
		-text => "\nPlease make sure you entered\nan appropriate Server Name.\n")
		->pack(-side => 'top') if (!$serverName);
	$errorWindow->Label(
		-text => "\nPlease make sure you entered\na valid Server IP Address.\n")
		->pack(-side => 'top') if ((!$serverIP) || !($serverIP =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/));
	$errorWindow->Label(
		-text => "\nPlease make sure you entered\na valid Server Port.\n")
		->pack(-side => 'top') if ((!$serverPort) || !($serverPort =~ /^\d+$/));
	my $closeButtonFrame = $errorWindow->Frame()
		->pack(-side => 'bottom', -pady => 10);
	my $closeButton = $closeButtonFrame->Button(
		-text => "Close", 
		-underline => 0, 
		-command => sub { 
			$errorWindow->destroy(); 
		}, 
		-width => 6)
		->pack(-side => 'top');
	$errorWindow->bind("<Alt-Key-c>" => sub { 
		$errorWindow->destroy(); 
	});
}
#---------------------------------------------------
# Deletes original server from hash, then creates it again with new
# server info.  Then writes the serverlist to file, and updates the
# main window
# usage: &saveServerInfo($serverToEdit, $serverName, $serverIP, $serverPort, $serverCheckType);
sub saveServerInfo {
	my ($serverToEdit, $serverName, $serverIP, $serverPort, $serverCheckType) = @_;

	delete $servers{$serverToEdit};
	delete $ports{$serverToEdit};

	$servers{$serverName} = $serverIP;
	$ports{$serverName} = $serverPort;
	$type{$serverName} = $serverCheckType;

	&writeServerList();
	&updateServerList();
}

#---------------------------------------------------
# Simply fires off an email notification that a server is down
# usage: $emailSent[$index] = &sendEmailNoti($server, $status);
sub sendEmailNoti {
	my ($server, $status) = @_;
	my $smtp = Net::SMTP->new($config{smtp_server});
# If the SMTP connection is made, send an email...
	if ($smtp) {
		$smtp->mail($config{sender_address});
		$smtp->recipient($config{recip_address});

		$smtp->data();
		$smtp->datasend("To: ".$config{recip_address}."\n");
		$smtp->datasend("Subject: Server Status - Server ".$server." ".$status."!\n");
		$smtp->datasend("\n");
		$smtp->datasend($server." (IP address ".$servers{$server});
		$smtp->datasend(", port ".$ports{$server}) if ($type{$server} eq "port");
		$smtp->datasend(") is ".$status." at ".localtime(time)."!\n");
		$smtp->datasend("\n");
		$smtp->dataend();

		$smtp->quit;
# ... otherwise display an error window
	} else {
		&errorWindow("smtp");
	}
# Even if sending the email fails, return 1 so that more attempts are not made
	return 1;
}

#---------------------------------------------------
# Just runs the clearServerList and drawServerList subroutines
# (mainly for the $mainWindow->repeat(); before MainLoop; above)
# usage: updateServerList($deleted);
sub updateServerList {
        my ($deleted) = @_;
	&clearServerList($deleted);
	&drawServerList();
}

#---------------------------------------------------
# Write the server status list to an HTML file,
# to be uploaded via FTP to a webserver
# usage: &writeHTML();
sub writeHTML {
        open(HTML, ">$configdir/status.html");
	select(HTML);
        print "<html>\n";
        print "<head>\n";
        print "  <title>Server Status v$version</title>\n";
        print "  <meta http-equiv=\"refresh\" content=\"60\">\n";
        print "</head>\n";
        print "<body>\n";
        print "<h3>Server Status v$version</h3>\n";
        print "<b>Last Updated:</b> ".localtime(time)."<p>\n";
        print "<table border=1 width=650>\n";
        print "  <tr>\n";
        print "    <td width=500><b>Server Name</b></td>\n";
        print "    <td width=75><b>Port</b></td>\n";
        print "    <td width=75><b>Status</b></td>\n";
        print "  </tr>\n";
        
        foreach my $server (sort(keys(%servers))) {
                print "  <tr>\n";
                print "    <td>$server";
		print " (ping)" if ($type{$server} eq "ping");
		print "</td>\n";
                print "    <td>$ports{$server}</td>\n";
                print "    <td>";
                if ($status{$server} == 1) {
                        print "Up";
                } else {
                        print "Down";
                }
                print "</td>\n";
                print "  </tr>\n";
        }
        
        print "</table>\n";
        print "</body>\n";
        print "</html>\n";
        select(STDOUT);
        close(HTML);
        
        &ftpHTML() if $config{useFTP};
}

#---------------------------------------------------
# Write the server list to the serverlist file
# Called when a server is added or deleted
# usage: &writeServerList();
sub writeServerList {
	open (SERVERLIST, ">$servercfg")
		|| die "Cannot open file: $servercfg: $!\n";
	print SERVERLIST "<opt>\n";
	foreach my $key (sort(keys %servers)) {
		print SERVERLIST "  <server address=\"$servers{$key}\" name=\"$key\" port=\"$ports{$key}\" type=\"$type{$key}\" />\n";
	}
	print SERVERLIST "</opt>\n";
	close (SERVERLIST);
}
#---------------------------------------------------
# End subroutines
#---------------------------------------------------

=head1 NAME

Server Status: Server monitor written in Perl\Tk

=head1 DESCRIPTION

Server Status is a server monitor that simply displays a list of servers
to monitor and displays whether they are alive on the network or not.

=head1 PREREQUISITE

uses Tk, IO::Socket, Net::SMTP, Net::FTP, Net::Ping

now requires AboutWindow.pm and ConfigFile.pm (included)

=head1 USAGE 

./status.pl

=head1 OSNAMES

Win32, Unix

=head1 SCRIPT CATEGORIES

GUI Programming, Networking

=cut
